from invenio_records_resources.services import RecordService
from invenio_records_resources.services import RecordService as InvenioRecordService


class DocumentsService(RecordService):
    """DocumentsRecord service."""



from flask import Blueprint


def create_blueprint_from_app_common(app):
    """Create  blueprint."""
    blueprint = app.extensions["common"].resource.as_blueprint()
    blueprint.record_once(init_create_blueprint_from_app_common)

    # calls record_once for all other functions starting with "init_addons_"
    # https://stackoverflow.com/questions/58785162/how-can-i-call-function-with-string-value-that-equals-to-function-name
    funcs = globals()
    funcs = [
        v
        for k, v in funcs.items()
        if k.startswith("init_addons_common") and callable(v)
    ]
    for func in funcs:
        blueprint.record_once(func)

    return blueprint


def init_create_blueprint_from_app_common(state):
    """Init app."""
    app = state.app
    ext = app.extensions["common"]

    # register service
    sregistry = app.extensions["invenio-records-resources"].registry
    sregistry.register(ext.service, service_id="common")

    # Register indexer
    if hasattr(ext.service, "indexer"):
        iregistry = app.extensions["invenio-indexer"].registry
        iregistry.register(ext.service.indexer, indexer_id="common")


def create_blueprint_from_app_commonExt(app):
    """Create -ext blueprint."""
    blueprint = Blueprint("common-ext", __name__, url_prefix="common")
    blueprint.record_once(init_create_blueprint_from_app_common)

    # calls record_once for all other functions starting with "init_app_addons_"
    # https://stackoverflow.com/questions/58785162/how-can-i-call-function-with-string-value-that-equals-to-function-name
    funcs = globals()
    funcs = [
        v
        for k, v in funcs.items()
        if k.startswith("init_app_addons_common") and callable(v)
    ]
    for func in funcs:
        blueprint.record_once(func)

    return blueprint



from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import RecordServiceConfig
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links

from nr_metadata.common.records.api import CommonRecord
from nr_metadata.common.services.records.permissions import CommonPermissionPolicy
from nr_metadata.common.services.records.schema import NRCommonRecordSchema
from nr_metadata.common.services.records.search import CommonSearchOptions


class CommonServiceConfig(RecordServiceConfig):
    """CommonRecord service config."""

    url_prefix = "/nr-metadata.common/"

    permission_policy_cls = CommonPermissionPolicy

    schema = NRCommonRecordSchema

    search = CommonSearchOptions

    record_cls = CommonRecord
    # todo should i leave this here?
    service_id = "common"

    components = [*RecordServiceConfig.components]

    model = "common"

    @property
    def links_item(self):
        return {
            "self": RecordLink("{self.url_prefix}{id}"),
        }

    @property
    def links_search(self):
        return pagination_links("{self.url_prefix}{?args*}")



from invenio_db import db
from invenio_records.models import RecordMetadataBase


class DocumentsMetadata(db.Model, RecordMetadataBase):
    """Model for DocumentsRecord metadata."""

    __tablename__ = "documents_metadata"

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}



import langcodes
from marshmallow import Schema, ValidationError, fields, validates

"""
Marshmallow schema for multilingual strings. Consider moving this file to a library, not generating
it for each project.
"""


class MultilingualSchema(Schema):
    lang = fields.String(required=True)
    value = fields.String(required=True)

    @validates("lang")
    def validate_lang(self, value):
        if value != "_" and not langcodes.Language.get(value).is_valid():
            raise ValidationError("Invalid language code")


class MultilingualUISchema(Schema):
    lang = fields.String(required=True)
    value = fields.String(required=True)

    @validates("lang")
    def validate_lang(self, value):
        if value != "_" and not langcodes.Language.get(value).is_valid():
            raise ValidationError("Invalid language code")



from flask_resources import BaseListSchema, MarshmallowSerializer
from flask_resources.serializers import JSONSerializer

from nr_metadata.documents.services.records.ui_schema import NRDocumentRecordUISchema


class DocumentsUIJSONSerializer(MarshmallowSerializer):
    """UI JSON serializer."""

    def __init__(self):
        """Initialise Serializer."""
        super().__init__(
            format_serializer_cls=JSONSerializer,
            object_schema_cls=NRDocumentRecordUISchema,
            list_schema_cls=BaseListSchema,
            schema_context={"object_key": "ui"},
        )



from nr_metadata.documents.resources.records.config import DocumentsResourceConfig
from nr_metadata.documents.resources.records.resource import DocumentsResource
from nr_metadata.documents.services.records.config import DocumentsServiceConfig
from nr_metadata.documents.services.records.service import DocumentsService

DOCUMENTS_RESOURCE_CONFIG_DOCUMENTS = DocumentsResourceConfig
DOCUMENTS_RESOURCE_CLASS_DOCUMENTS = DocumentsResource
DOCUMENTS_SERVICE_CONFIG_DOCUMENTS = DocumentsServiceConfig
DOCUMENTS_SERVICE_CLASS_DOCUMENTS = DocumentsService



from invenio_records_permissions import RecordPermissionPolicy
from invenio_records_permissions.generators import AnyUser, SystemProcess


class DocumentsPermissionPolicy(RecordPermissionPolicy):
    """nr_metadata.documents.records.api.DocumentsRecord permissions."""

    can_search = [SystemProcess(), AnyUser()]
    can_read = [SystemProcess(), AnyUser()]
    can_create = [SystemProcess()]
    can_update = [SystemProcess()]
    can_delete = [SystemProcess()]
    can_manage = [SystemProcess()]



from invenio_records_resources.services import SearchOptions as InvenioSearchOptions

from . import facets


def _(x):
    """Identity function for string extraction."""
    return x


class DocumentsSearchOptions(InvenioSearchOptions):
    """DocumentsRecord search options."""

    facets = {
        "metadata_thesis_dateDefended": facets.metadata_thesis_dateDefended,
        "metadata_thesis_defended": facets.metadata_thesis_defended,
        "metadata_thesis_degreeGrantor_id": facets.metadata_thesis_degreeGrantor_id,
        "metadata_thesis_degreeGrantor_hierarchy_parent": (
            facets.metadata_thesis_degreeGrantor_hierarchy_parent
        ),
        "metadata_thesis_degreeGrantor_hierarchy_level": (
            facets.metadata_thesis_degreeGrantor_hierarchy_level
        ),
        "metadata_thesis_degreeGrantor_hierarchy_ancestors": (
            facets.metadata_thesis_degreeGrantor_hierarchy_ancestors
        ),
        "metadata_thesis_degreeGrantor__version": (
            facets.metadata_thesis_degreeGrantor__version
        ),
        "metadata_thesis_studyFields": facets.metadata_thesis_studyFields,
        "metadata_collection": facets.metadata_collection,
        "metadata_title_keyword": facets.metadata_title_keyword,
        "metadata_additionalTitles_title_lang": (
            facets.metadata_additionalTitles_title_lang
        ),
        "metadata_additionalTitles_title_value_keyword": (
            facets.metadata_additionalTitles_title_value_keyword
        ),
        "metadata_additionalTitles_titleType": (
            facets.metadata_additionalTitles_titleType
        ),
        "metadata_creators_affiliations_id": facets.metadata_creators_affiliations_id,
        "metadata_creators_affiliations__version": (
            facets.metadata_creators_affiliations__version
        ),
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_creators_fullName": facets.metadata_creators_fullName,
        "metadata_creators_authorityIdentifiers_identifier": (
            facets.metadata_creators_authorityIdentifiers_identifier
        ),
        "metadata_creators_authorityIdentifiers_scheme": (
            facets.metadata_creators_authorityIdentifiers_scheme
        ),
        "metadata_contributors_role_id": facets.metadata_contributors_role_id,
        "metadata_contributors_role__version": (
            facets.metadata_contributors_role__version
        ),
        "metadata_contributors_affiliations_id": (
            facets.metadata_contributors_affiliations_id
        ),
        "metadata_contributors_affiliations__version": (
            facets.metadata_contributors_affiliations__version
        ),
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_contributors_fullName": facets.metadata_contributors_fullName,
        "metadata_contributors_authorityIdentifiers_identifier": (
            facets.metadata_contributors_authorityIdentifiers_identifier
        ),
        "metadata_contributors_authorityIdentifiers_scheme": (
            facets.metadata_contributors_authorityIdentifiers_scheme
        ),
        "metadata_resourceType_id": facets.metadata_resourceType_id,
        "metadata_resourceType__version": facets.metadata_resourceType__version,
        "metadata_dateAvailable": facets.metadata_dateAvailable,
        "metadata_dateModified": facets.metadata_dateModified,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_subject_lang": facets.metadata_subjects_subject_lang,
        "metadata_subjects_subject_value_keyword": (
            facets.metadata_subjects_subject_value_keyword
        ),
        "metadata_subjects_valueURI": facets.metadata_subjects_valueURI,
        "metadata_subjects_classificationCode": (
            facets.metadata_subjects_classificationCode
        ),
        "metadata_subjectCategories_id": facets.metadata_subjectCategories_id,
        "metadata_subjectCategories__version": (
            facets.metadata_subjectCategories__version
        ),
        "metadata_languages_id": facets.metadata_languages_id,
        "metadata_languages__version": facets.metadata_languages__version,
        "metadata_abstract_lang": facets.metadata_abstract_lang,
        "metadata_abstract_value_keyword": facets.metadata_abstract_value_keyword,
        "metadata_methods_lang": facets.metadata_methods_lang,
        "metadata_methods_value_keyword": facets.metadata_methods_value_keyword,
        "metadata_technicalInfo_lang": facets.metadata_technicalInfo_lang,
        "metadata_technicalInfo_value_keyword": (
            facets.metadata_technicalInfo_value_keyword
        ),
        "metadata_rights_id": facets.metadata_rights_id,
        "metadata_rights__version": facets.metadata_rights__version,
        "metadata_accessRights_id": facets.metadata_accessRights_id,
        "metadata_accessRights__version": facets.metadata_accessRights__version,
        "metadata_relatedItems_itemCreators_affiliations_id": (
            facets.metadata_relatedItems_itemCreators_affiliations_id
        ),
        "metadata_relatedItems_itemCreators_affiliations__version": (
            facets.metadata_relatedItems_itemCreators_affiliations__version
        ),
        "metadata_relatedItems_itemCreators_nameType": (
            facets.metadata_relatedItems_itemCreators_nameType
        ),
        "metadata_relatedItems_itemCreators_fullName": (
            facets.metadata_relatedItems_itemCreators_fullName
        ),
        "metadata_relatedItems_itemCreators_authorityIdentifiers_identifier": (
            facets.metadata_relatedItems_itemCreators_authorityIdentifiers_identifier
        ),
        "metadata_relatedItems_itemCreators_authorityIdentifiers_scheme": (
            facets.metadata_relatedItems_itemCreators_authorityIdentifiers_scheme
        ),
        "metadata_relatedItems_itemContributors_role_id": (
            facets.metadata_relatedItems_itemContributors_role_id
        ),
        "metadata_relatedItems_itemContributors_role__version": (
            facets.metadata_relatedItems_itemContributors_role__version
        ),
        "metadata_relatedItems_itemContributors_affiliations_id": (
            facets.metadata_relatedItems_itemContributors_affiliations_id
        ),
        "metadata_relatedItems_itemContributors_affiliations__version": (
            facets.metadata_relatedItems_itemContributors_affiliations__version
        ),
        "metadata_relatedItems_itemContributors_nameType": (
            facets.metadata_relatedItems_itemContributors_nameType
        ),
        "metadata_relatedItems_itemContributors_fullName": (
            facets.metadata_relatedItems_itemContributors_fullName
        ),
        "metadata_relatedItems_itemContributors_authorityIdentifiers_identifier": (
            facets.metadata_relatedItems_itemContributors_authorityIdentifiers_identifier
        ),
        "metadata_relatedItems_itemContributors_authorityIdentifiers_scheme": (
            facets.metadata_relatedItems_itemContributors_authorityIdentifiers_scheme
        ),
        "metadata_relatedItems_itemPIDs_identifier": (
            facets.metadata_relatedItems_itemPIDs_identifier
        ),
        "metadata_relatedItems_itemPIDs_scheme": (
            facets.metadata_relatedItems_itemPIDs_scheme
        ),
        "metadata_relatedItems_itemURL": facets.metadata_relatedItems_itemURL,
        "metadata_relatedItems_itemYear": facets.metadata_relatedItems_itemYear,
        "metadata_relatedItems_itemVolume": facets.metadata_relatedItems_itemVolume,
        "metadata_relatedItems_itemIssue": facets.metadata_relatedItems_itemIssue,
        "metadata_relatedItems_itemStartPage": (
            facets.metadata_relatedItems_itemStartPage
        ),
        "metadata_relatedItems_itemEndPage": facets.metadata_relatedItems_itemEndPage,
        "metadata_relatedItems_itemPublisher": (
            facets.metadata_relatedItems_itemPublisher
        ),
        "metadata_relatedItems_itemRelationType_id": (
            facets.metadata_relatedItems_itemRelationType_id
        ),
        "metadata_relatedItems_itemRelationType__version": (
            facets.metadata_relatedItems_itemRelationType__version
        ),
        "metadata_relatedItems_itemResourceType_id": (
            facets.metadata_relatedItems_itemResourceType_id
        ),
        "metadata_relatedItems_itemResourceType__version": (
            facets.metadata_relatedItems_itemResourceType__version
        ),
        "metadata_fundingReferences_projectID": (
            facets.metadata_fundingReferences_projectID
        ),
        "metadata_fundingReferences_funder_id": (
            facets.metadata_fundingReferences_funder_id
        ),
        "metadata_fundingReferences_funder__version": (
            facets.metadata_fundingReferences_funder__version
        ),
        "metadata_version": facets.metadata_version,
        "metadata_geoLocations_geoLocationPlace": (
            facets.metadata_geoLocations_geoLocationPlace
        ),
        "metadata_geoLocations_geoLocationPoint_pointLongitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLongitude
        ),
        "metadata_geoLocations_geoLocationPoint_pointLatitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLatitude
        ),
        "metadata_accessibility_lang": facets.metadata_accessibility_lang,
        "metadata_accessibility_value_keyword": (
            facets.metadata_accessibility_value_keyword
        ),
        "metadata_series_seriesTitle": facets.metadata_series_seriesTitle,
        "metadata_series_seriesVolume": facets.metadata_series_seriesVolume,
        "metadata_externalLocation_externalLocationURL": (
            facets.metadata_externalLocation_externalLocationURL
        ),
        "metadata_originalRecord": facets.metadata_originalRecord,
        "metadata_objectIdentifiers_identifier": (
            facets.metadata_objectIdentifiers_identifier
        ),
        "metadata_objectIdentifiers_scheme": facets.metadata_objectIdentifiers_scheme,
        "metadata_systemIdentifiers_identifier": (
            facets.metadata_systemIdentifiers_identifier
        ),
        "metadata_systemIdentifiers_scheme": facets.metadata_systemIdentifiers_scheme,
        "metadata_events_eventDate": facets.metadata_events_eventDate,
        "metadata_events_eventLocation_place": (
            facets.metadata_events_eventLocation_place
        ),
        "metadata_events_eventLocation_country_id": (
            facets.metadata_events_eventLocation_country_id
        ),
        "metadata_events_eventLocation_country__version": (
            facets.metadata_events_eventLocation_country__version
        ),
        "_id": facets._id,
        "created": facets.created,
        "updated": facets.updated,
        "_schema": facets._schema,
    }
    sort_options = {
        **InvenioSearchOptions.sort_options,
        "bestmatch": dict(
            title=_("Best match"),
            fields=["_score"],  # ES defaults to desc on `_score` field
        ),
        "newest": dict(
            title=_("Newest"),
            fields=["-created"],
        ),
        "oldest": dict(
            title=_("Oldest"),
            fields=["created"],
        ),
    }



import marshmallow as ma
from marshmallow import validate
from idutils import normalize_pid, to_url


class NRIdentifierUISchema(ma.Schema):
    scheme = ma.fields.String(
        required=True,
    )
    identifier = ma.fields.String(required=True)

    @ma.post_dump
    def add_url(self, value):
        url = to_url(value["identifier"], value["scheme"].lower(), url_scheme="https")
        if url:
            value["url"] = url
        return value


class NRObjectIdentifierUISchema(NRIdentifierUISchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(["DOI", "Handle", "ISBN", "ISSN", "RIV"])
        ],  # RIV is not normalized, others are
    )


class NRAuthorityIdentifierUISchema(NRIdentifierUISchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                [
                    "orcid",  # normalized
                    "scopusID",
                    "researcherID",
                    "czenasAutID",
                    "vedidk",
                    "institutionalID",
                    "ISNI",
                    "ROR",
                    "ICO",
                    "DOI",  # normalized
                ]
            )
        ],
    )


class NRSystemIdentifierUISchema(NRIdentifierUISchema):
    scheme = ma.fields.String(
        required=True,
        validate=[
            validate.OneOf(
                ["nusl", "nuslOAI", "originalRecordOAI", "catalogueSysNo", "nrOAI"]
            )
        ],
    )



from invenio_db import db
from invenio_records.models import RecordMetadataBase


class CommonMetadata(db.Model, RecordMetadataBase):
    """Model for CommonRecord metadata."""

    __tablename__ = "common_metadata"

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}
